Package: cjoint
Version: 2.1.3
Date: 2026-05-18
Title: AMCE Estimator for Conjoint Experiments
Authors@R: c(
    person("Soubhik", "Barari", role = "aut"),
    person("Elissa", "Berwick", role = "aut"),
    person("Jens", "Hainmueller", role = "aut"),
    person("Daniel", "Hopkins", role = "aut"),
    person("Sean", "Liu", role = "aut"),
    person("Anton", "Strezhnev", role = c("aut", "cre"),
           email = "strezhnev@wisc.edu"),
    person("Teppei", "Yamamoto", role = "aut"))
Depends: R (>= 2.10), sandwich, lmtest, ggplot2, survey, Matrix
Imports: stats, utils, methods, grDevices, DT, shiny, shinyjs
Suggests: testthat
Description: An R implementation of the Average Marginal Component-specific
    Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto
    T. (2014) <DOI:10.1093/pan/mpt024> Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional
    Choices via Stated Preference Experiments. Political Analysis 22(1):1-30.
License: GPL (>= 2)
RoxygenNote: 6.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-18 19:10:22 UTC; anton
Author: Soubhik Barari [aut],
  Elissa Berwick [aut],
  Jens Hainmueller [aut],
  Daniel Hopkins [aut],
  Sean Liu [aut],
  Anton Strezhnev [aut, cre],
  Teppei Yamamoto [aut]
Maintainer: Anton Strezhnev <strezhnev@wisc.edu>
Repository: CRAN
Date/Publication: 2026-05-19 05:30:02 UTC
Built: R 4.5.2; ; 2026-05-19 06:35:45 UTC; unix
