Package: cjoint
Version: 2.1.1
Date: 2023-08-21
Title: AMCE Estimator for Conjoint Experiments
Author: Soubhik Barari, Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Sean Liu, Anton Strezhnev, Teppei Yamamoto
Maintainer: Anton Strezhnev <astrezhnev@uchicago.edu>
Depends: R (>= 2.10), sandwich, lmtest, ggplot2, survey, Matrix
Imports: stats, utils, methods, DT, shiny, shinyjs
Suggests: testthat
Description: An R implementation of the Average Marginal Component-specific
    Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto
    T. (2014) <DOI:10.1093/pan/mpt024> Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional
    Choices via Stated Preference Experiments. Political Analysis 22(1):1-30.
License: GPL (>= 2)
RoxygenNote: 6.1.0
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2023-08-21 20:00:11 UTC; anton
Repository: CRAN
Date/Publication: 2023-08-22 07:50:07 UTC
Built: R 4.2.0; ; 2023-08-27 03:40:38 UTC; unix
