# Generated by roxygen2: do not edit by hand

S3method(plot,amce)
S3method(print,summary.amce)
S3method(summary,amce)
export(amce)
export(makeDesign)
export(plot.amce)
export(print.summary.amce)
export(read.qualtrics)
export(read.with.qualtRics)
export(summary.amce)
export(view)
import(Matrix)
import(ggplot2)
import(lmtest)
import(sandwich)
import(shiny)
import(survey)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(methods,Quote)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,reshape)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,View)
importFrom(utils,data)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)
