# cjoint: An R Package for Estimating Average Marginal Component-specific Effects for Conjoint Survey Experiments

## Description
This package allows researchers to estimate the causal effects of attributes in conjoint survey experiments. It implements the Average Marginal Component-specific Effects (AMCE) estimator presented in Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30

## Details
  Package: `cjoint`
  Type: `R Package`
  Version: 2.1.3
  Date: 2026-05-18
  License: GPL (>= 2)

## Authors: 
Soubhik Barari, Elissa Berwick, Jens Hainmueller, Daniel Hopkins, Sean Liu, Anton Strezhnev, Teppei Yamamoto

## Maintainer: 
Anton Strezhnev <strezhnev@wisc.edu>

## References:
Hainmueller, J., Hopkins, D., and Yamamoto T. (2014) Causal Inference in Conjoint Analysis: Understanding Multi-Dimensional Choices via Stated Preference Experiments. Political Analysis 22(1):1-30

## Installation Guide:

To install the `Github` version of the `cjoint` package, use the following command:

`remotes::install_github("astrezhnev/cjoint")`